<?php

/**
 * Version 1.1
 */
class BaoKimPayment {

  // URL checkout
  private $baokim_url = '';
  // Merchant ID
  private $merchant_id = '';
  // Secure pass
  private $secure_pass = '';

  /**
   * Construct
   */
  function __construct($baokim_url = '', $merchant_id = '', $secure_pass = '') {
    $this->baokim_url = $baokim_url;
    $this->merchant_id = $merchant_id;
    $this->secure_pass = $secure_pass;
  }

  /**
   * Create standard BaoKim URL
   */
  private function createURL($urlBaoKim, $params) {
    ksort($params);

    $params['checksum'] = hash_hmac('SHA1', implode('', $params), $this->secure_pass);

    //Kiểm tra  biến $redirect_url xem có '?' không, nếu không có thì bổ sung vào
    $redirect_url = $urlBaoKim;
    if (strpos($redirect_url, '?') === false) {
      $redirect_url .= '?';
    } else if (substr($redirect_url, strlen($redirect_url) - 1, 1) != '?' && strpos($redirect_url, '&') === false) {
      // Nếu biến $redirect_url có '?' nhưng không kết thúc bằng '?' và có chứa dấu '&' thì bổ sung vào cuối
      $redirect_url .= '&';
    }

    // Tạo đoạn url chứa tham số
    $url_params = '';
    foreach ($params as $key => $value) {
      if ($url_params == '')
        $url_params .= $key . '=' . urlencode($value);
      else
        $url_params .= '&' . $key . '=' . urlencode($value);
    }
    return $redirect_url . $url_params;
  }

  /**
   * Hàm xây dựng url chuyển đến BaoKim.vn thực hiện thanh toán, trong đó có tham số mã hóa (còn gọi là public key)
   * @param $order_id Mã đơn hàng
   * @param $business Email tài khoản người bán
   * @param $total_amount Giá trị đơn hàng
   * @param $shipping_fee Phí vận chuyển
   * @param $tax_fee Thuế
   * @param $order_description  Mô tả đơn hàng
   * @param $url_success Url trả về khi thanh toán thành công
   * @param $url_cancel Url trả về khi hủy thanh toán
   * @param $url_detail Url chi tiết đơn hàng
   * @return url cần tạo
   *
   */
  public function createRequestUrl(
  $order_id
  , $business
  , $total_amount
  , $shipping_fee
  , $tax_fee
  , $order_description
  , $url_success
  , $url_cancel
  , $url_detail
  , $payer_name = null
  , $payer_email = null
  , $payer_phone_no = null
  , $shipping_address = null
  , $currency = 'VND') {

    // Mảng các tham số chuyển tới baokim.vn
    $params = array(
      'merchant_id' => strval($this->merchant_id),
      'order_id' => strval($order_id),
      'business' => strval($business),
      'total_amount' => strval($total_amount),
      'shipping_fee' => strval($shipping_fee),
      'tax_fee' => strval($tax_fee),
      'order_description' => strval($order_description),
      'url_success' => strtolower($url_success),
      'url_cancel' => strtolower($url_cancel),
      'url_detail' => strtolower($url_detail),
      'payer_name' => strval($payer_name),
      'payer_email' => strval($payer_email),
      'payer_phone_no' => strval($payer_phone_no),
      'shipping_address' => strval($shipping_address),
      'currency' => strval($currency)
    );

    return $this->createURL($this->baokim_url, $params);
  }

  /**
   * Hàm thực hiện xác minh tính chính xác thông tin trả về từ BaoKim.vn
   * @param $url_params chứa tham số trả về trên url
   * @return true nếu thông tin là chính xác, false nếu thông tin không chính xác
   */
  public function verifyResponseUrl($url_params = array()) {
    if (empty($url_params['checksum'])) {
      // return "invalid parameters: checksum is missing";
      watchdog('commerce_baokim', "FAILED: invalid parameters: checksum is missing");
      return FALSE;
    }

    $checksum = $url_params['checksum'];
    unset($url_params['checksum']);

    ksort($url_params);

    if (strcasecmp($checksum, hash_hmac('SHA1', implode('', $url_params), $this->secure_pass)) === 0)
      return TRUE;
    else {
      watchdog('commerce_baokim', "FAILED: Unknown");
      return FALSE;
    }
  }

  /**
   * Create verify payment URL
   */
  public function createVerifyPaymentURL($urlVerify, $params) {
    return $this->createURL($urlVerify, $params);
  }

}
